/* eslint-disable @typescript-eslint/no-non-null-assertion */ "use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "TypescriptCompiler", {
    enumerable: true,
    get: ()=>TypescriptCompiler
});
const _extends = require("@swc/helpers/lib/_extends.js").default;
const _interopRequireDefault = require("@swc/helpers/lib/_interop_require_default.js").default;
const _typescript = /*#__PURE__*/ _interopRequireDefault(require("typescript"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _utilities = require("@module-federation/utilities");
const _caching = require("./Caching");
let TypescriptCompiler = class TypescriptCompiler {
    generateDeclarationFiles(exposedComponents, additionalFilesToCompile = []) {
        const exposeSrcToDestMap = {};
        const normalizedExposedComponents = this.normalizeFiles(Object.entries(exposedComponents), ([exposeDest, exposeSrc])=>{
            const pathWithExt = this.getNormalizedPathWithExt(exposeSrc);
            exposeSrcToDestMap[pathWithExt] = exposeDest;
            return pathWithExt;
        });
        const normalizedAdditionalFiles = this.normalizeFiles(additionalFilesToCompile, this.getNormalizedPathWithExt.bind(this));
        const host = this.createHost(exposeSrcToDestMap);
        const program = _typescript.default.createProgram([
            ...normalizedAdditionalFiles,
            ...normalizedExposedComponents
        ], this.compilerOptions, host);
        const { diagnostics , emitSkipped  } = program.emit();
        if (!emitSkipped) {
            return this.tsDefinitionFilesObj;
        }
        diagnostics.forEach(this.reportCompileDiagnostic.bind(this));
        throw new Error('something went wrong generating declaration files');
    }
    normalizeFiles(files, mapFn) {
        return files.map(mapFn).filter((entry)=>/\.tsx?$/.test(entry));
    }
    getNormalizedPathWithExt(exposeSrc) {
        const cwd = this.options.webpackCompilerOptions.context || process.cwd();
        const [rootDir, entry] = exposeSrc.split(/\/(?=[^/]+$)/);
        const normalizedRootDir = _path.default.resolve(cwd, rootDir);
        const filenameWithExt = this.getFilenameWithExtension(normalizedRootDir, entry);
        const pathWithExt = _path.default.resolve(normalizedRootDir, filenameWithExt);
        return pathWithExt;
    }
    createHost(exposeSrcToDestMap) {
        const host = _typescript.default.createCompilerHost(this.compilerOptions);
        const originalWriteFile = host.writeFile;
        const rewritePathsWithExposedFederatedModules = (sourceFilename)=>{
            const destFile = exposeSrcToDestMap[sourceFilename];
            return destFile && _path.default.join(this.compilerOptions.outDir, `${destFile}.d.ts`);
        };
        host.writeFile = (filepath, text, writeOrderByteMark, onError, sourceFiles, data)=>{
            // for exposes: { "./expose/path": "path/to/file" }
            // force typescript to write compiled output to "@mf-typescript/expose/path"
            const sourceFilename = (sourceFiles == null ? void 0 : sourceFiles[0].fileName) || '';
            var ref;
            // Try to rewrite the path with exposed federated modules,
            // failing so, use the default filepath emitted by TS Compiler.
            // This second case is valid for 'additionalFileToCompiler' added through Plugin Options.
            const normalizedFilepath = (ref = rewritePathsWithExposedFederatedModules(sourceFilename)) != null ? ref : filepath;
            this.tsDefinitionFilesObj[normalizedFilepath] = text;
            originalWriteFile(normalizedFilepath, text, writeOrderByteMark, onError, sourceFiles, data);
        };
        return host;
    }
    reportCompileDiagnostic(diagnostic) {
        const { line  } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
        this.logger.log('TS Error', diagnostic.code, ':', _typescript.default.flattenDiagnosticMessageText(diagnostic.messageText, _typescript.default.sys.newLine));
        this.logger.log('         at', `${diagnostic.file.fileName}:${line + 1}`, _typescript.default.sys.newLine // '\n'
        );
    }
    getTSConfigCompilerOptions() {
        const context = this.options.webpackCompilerOptions.context;
        const tsconfigPath = _typescript.default.findConfigFile(context, _typescript.default.sys.fileExists, 'tsconfig.json');
        if (!tsconfigPath) {
            this.logger.error('ERROR: Could not find a valid tsconfig.json');
            process.exit(1);
        }
        const readResult = _typescript.default.readConfigFile(tsconfigPath, _typescript.default.sys.readFile);
        const configContent = _typescript.default.parseJsonConfigFileContent(readResult.config, _typescript.default.sys, context);
        return configContent.options;
    }
    getFilenameWithExtension(rootDir, entry) {
        // Check path exists and it's a directory
        if (!_fs.default.existsSync(rootDir) || !_fs.default.lstatSync(rootDir).isDirectory()) {
            throw new Error('rootDir must be a directory');
        }
        let filename;
        try {
            // Try to resolve exposed component using index
            const files = _caching.TypesCache.getFsFiles(_path.default.join(rootDir, entry));
            filename = files == null ? void 0 : files.find((file)=>file.split('.')[0] === 'index');
            if (!filename) {
                throw new Error(`filename ${filename} not found`);
            }
            return `${entry}/${filename}`;
        } catch (err) {
            const files1 = _caching.TypesCache.getFsFiles(rootDir);
            // Handle case where directory contains similar filenames
            // or where a filename like `Component.base.tsx` is used
            filename = files1 == null ? void 0 : files1.find((file)=>{
                const baseFile = _path.default.basename(file, _path.default.extname(file));
                const baseEntry = _path.default.basename(entry, _path.default.extname(entry));
                return baseFile === baseEntry;
            });
            if (!filename) {
                throw new Error(`filename ${filename} not found`);
            }
            return filename;
        }
    }
    constructor(options){
        this.options = options;
        this.tsDefinitionFilesObj = {};
        this.logger = _utilities.Logger.getLogger();
        const tsConfigCompilerOptions = this.getTSConfigCompilerOptions();
        this.compilerOptions = _extends({}, tsConfigCompilerOptions, options.tsCompilerOptions);
    }
};

//# sourceMappingURL=TypescriptCompiler.js.map